#ifndef QT_WAYLAND_HAWAII
#define QT_WAYLAND_HAWAII

#include "wayland-hawaii-client-protocol.h"
#include <QByteArray>
#include <QString>

QT_BEGIN_NAMESPACE

namespace QtWayland {
    class  hawaii_shell
    {
    public:
        hawaii_shell(struct ::wl_registry *registry, int id);
        hawaii_shell(struct ::hawaii_shell *object);
        hawaii_shell();

        virtual ~hawaii_shell();

        void init(struct ::wl_registry *registry, int id);
        void init(struct ::hawaii_shell *object);

        struct ::hawaii_shell *object() { return m_hawaii_shell; }
        const struct ::hawaii_shell *object() const { return m_hawaii_shell; }

        bool isInitialized() const;

        enum cursor {
            cursor_none = 0,
            cursor_resize_top = 1,
            cursor_resize_bottom = 2,
            cursor_arrow = 3,
            cursor_resize_left = 4,
            cursor_resize_top_left = 5,
            cursor_resize_bottom_left = 6,
            cursor_move = 7,
            cursor_resize_right = 8,
            cursor_resize_top_right = 9,
            cursor_resize_bottom_right = 10,
            cursor_busy = 11
        };

        enum window_state {
            window_state_inactive = 0,
            window_state_active = 1,
            window_state_minimized = 2,
            window_state_maximized = 4,
            window_state_fullscreen = 8
        };

        void add_trusted_client(int32_t fd, const QString &interface);
        struct ::hawaii_key_binding *add_key_binding(uint32_t key, uint32_t modifiers);
        void set_background(struct ::wl_output *output, struct ::wl_surface *surface);
        void set_desktop(struct ::wl_output *output, struct ::wl_surface *surface);
        void set_config_surface(struct ::wl_output *output, struct ::wl_surface *surface);
        void set_overlay(struct ::wl_surface *surface);
        void set_lock_surface(struct ::wl_surface *surface);
        void set_grab_surface(struct ::wl_surface *surface);
        void set_position(struct ::wl_surface *surface, int32_t x, int32_t y);
        void quit();
        void lock();
        void unlock();
        void desktop_ready();
        void minimize_windows();
        void restore_windows();
        void add_workspace();
        void select_workspace(struct ::hawaii_workspace *workspace);

    protected:
        virtual void shell_loaded();
        virtual void shell_configure(struct ::wl_surface *surface, int32_t width, int32_t height);
        virtual void shell_prepare_lock_surface();
        virtual void shell_grab_cursor(uint32_t cursor);
        virtual void shell_window_mapped(struct ::hawaii_window *id, const QString &title, const QString &identifier, int32_t state);
        virtual void shell_window_switching_started();
        virtual void shell_window_switching_finished();
        virtual void shell_window_switched(struct ::hawaii_window *window);
        virtual void shell_workspace_added(struct ::hawaii_workspace *workspace, int32_t active);

    private:
        void init_listener();
        static const struct hawaii_shell_listener m_hawaii_shell_listener;
        static void handle_loaded(
            void *data,
            struct ::hawaii_shell *object);
        static void handle_configure(
            void *data,
            struct ::hawaii_shell *object,
            struct ::wl_surface *surface,
            int32_t width,
            int32_t height);
        static void handle_prepare_lock_surface(
            void *data,
            struct ::hawaii_shell *object);
        static void handle_grab_cursor(
            void *data,
            struct ::hawaii_shell *object,
            uint32_t cursor);
        static void handle_window_mapped(
            void *data,
            struct ::hawaii_shell *object,
            struct ::hawaii_window *id,
            const char *title,
            const char *identifier,
            int32_t state);
        static void handle_window_switching_started(
            void *data,
            struct ::hawaii_shell *object);
        static void handle_window_switching_finished(
            void *data,
            struct ::hawaii_shell *object);
        static void handle_window_switched(
            void *data,
            struct ::hawaii_shell *object,
            struct ::hawaii_window *window);
        static void handle_workspace_added(
            void *data,
            struct ::hawaii_shell *object,
            struct ::hawaii_workspace *workspace,
            int32_t active);
        struct ::hawaii_shell *m_hawaii_shell;
    };

    class  hawaii_panel_manager
    {
    public:
        hawaii_panel_manager(struct ::wl_registry *registry, int id);
        hawaii_panel_manager(struct ::hawaii_panel_manager *object);
        hawaii_panel_manager();

        virtual ~hawaii_panel_manager();

        void init(struct ::wl_registry *registry, int id);
        void init(struct ::hawaii_panel_manager *object);

        struct ::hawaii_panel_manager *object() { return m_hawaii_panel_manager; }
        const struct ::hawaii_panel_manager *object() const { return m_hawaii_panel_manager; }

        bool isInitialized() const;

        struct ::hawaii_panel *set_panel(struct ::wl_surface *surface);

    private:
        struct ::hawaii_panel_manager *m_hawaii_panel_manager;
    };

    class  hawaii_panel
    {
    public:
        hawaii_panel(struct ::wl_registry *registry, int id);
        hawaii_panel(struct ::hawaii_panel *object);
        hawaii_panel();

        virtual ~hawaii_panel();

        void init(struct ::wl_registry *registry, int id);
        void init(struct ::hawaii_panel *object);

        struct ::hawaii_panel *object() { return m_hawaii_panel; }
        const struct ::hawaii_panel *object() const { return m_hawaii_panel; }

        bool isInitialized() const;

        enum edge {
            edge_left = 0,
            edge_top = 1,
            edge_right = 2,
            edge_bottom = 3
        };

        enum alignment {
            alignment_left = 0,
            alignment_center = 1,
            alignment_right = 2
        };

        void set_alignment(uint32_t alignment);
        void set_offset(uint32_t offset);
        void set_thickness(uint32_t thickness);
        void set_length(uint32_t length);
        void set_min_length(uint32_t min_length);
        void set_max_length(uint32_t max_length);
        void dock(uint32_t edge, struct ::wl_output *output);

    protected:
        virtual void panel_docked();

    private:
        void init_listener();
        static const struct hawaii_panel_listener m_hawaii_panel_listener;
        static void handle_docked(
            void *data,
            struct ::hawaii_panel *object);
        struct ::hawaii_panel *m_hawaii_panel;
    };

    class  hawaii_shell_surface
    {
    public:
        hawaii_shell_surface(struct ::wl_registry *registry, int id);
        hawaii_shell_surface(struct ::hawaii_shell_surface *object);
        hawaii_shell_surface();

        virtual ~hawaii_shell_surface();

        void init(struct ::wl_registry *registry, int id);
        void init(struct ::hawaii_shell_surface *object);

        struct ::hawaii_shell_surface *object() { return m_hawaii_shell_surface; }
        const struct ::hawaii_shell_surface *object() const { return m_hawaii_shell_surface; }

        bool isInitialized() const;

        struct ::hawaii_popup_surface *set_popup(struct ::wl_surface *parent, struct ::wl_surface *surface, int32_t x, int32_t y);
        void set_dialog(struct ::wl_output *output, struct ::wl_surface *surface);

    private:
        struct ::hawaii_shell_surface *m_hawaii_shell_surface;
    };

    class  hawaii_popup_surface
    {
    public:
        hawaii_popup_surface(struct ::wl_registry *registry, int id);
        hawaii_popup_surface(struct ::hawaii_popup_surface *object);
        hawaii_popup_surface();

        virtual ~hawaii_popup_surface();

        void init(struct ::wl_registry *registry, int id);
        void init(struct ::hawaii_popup_surface *object);

        struct ::hawaii_popup_surface *object() { return m_hawaii_popup_surface; }
        const struct ::hawaii_popup_surface *object() const { return m_hawaii_popup_surface; }

        bool isInitialized() const;

    protected:
        virtual void popup_surface_popup_done();

    private:
        void init_listener();
        static const struct hawaii_popup_surface_listener m_hawaii_popup_surface_listener;
        static void handle_popup_done(
            void *data,
            struct ::hawaii_popup_surface *object);
        struct ::hawaii_popup_surface *m_hawaii_popup_surface;
    };

    class  hawaii_key_binding
    {
    public:
        hawaii_key_binding(struct ::wl_registry *registry, int id);
        hawaii_key_binding(struct ::hawaii_key_binding *object);
        hawaii_key_binding();

        virtual ~hawaii_key_binding();

        void init(struct ::wl_registry *registry, int id);
        void init(struct ::hawaii_key_binding *object);

        struct ::hawaii_key_binding *object() { return m_hawaii_key_binding; }
        const struct ::hawaii_key_binding *object() const { return m_hawaii_key_binding; }

        bool isInitialized() const;

    protected:
        virtual void key_binding_triggered();

    private:
        void init_listener();
        static const struct hawaii_key_binding_listener m_hawaii_key_binding_listener;
        static void handle_triggered(
            void *data,
            struct ::hawaii_key_binding *object);
        struct ::hawaii_key_binding *m_hawaii_key_binding;
    };

    class  hawaii_window
    {
    public:
        hawaii_window(struct ::wl_registry *registry, int id);
        hawaii_window(struct ::hawaii_window *object);
        hawaii_window();

        virtual ~hawaii_window();

        void init(struct ::wl_registry *registry, int id);
        void init(struct ::hawaii_window *object);

        struct ::hawaii_window *object() { return m_hawaii_window; }
        const struct ::hawaii_window *object() const { return m_hawaii_window; }

        bool isInitialized() const;

        void set_state(int32_t state);
        void close();

    protected:
        virtual void window_title_changed(const QString &title);
        virtual void window_identifier_changed(const QString &identifier);
        virtual void window_state_changed(int32_t state);
        virtual void window_unmapped();

    private:
        void init_listener();
        static const struct hawaii_window_listener m_hawaii_window_listener;
        static void handle_title_changed(
            void *data,
            struct ::hawaii_window *object,
            const char *title);
        static void handle_identifier_changed(
            void *data,
            struct ::hawaii_window *object,
            const char *identifier);
        static void handle_state_changed(
            void *data,
            struct ::hawaii_window *object,
            int32_t state);
        static void handle_unmapped(
            void *data,
            struct ::hawaii_window *object);
        struct ::hawaii_window *m_hawaii_window;
    };

    class  hawaii_workspace
    {
    public:
        hawaii_workspace(struct ::wl_registry *registry, int id);
        hawaii_workspace(struct ::hawaii_workspace *object);
        hawaii_workspace();

        virtual ~hawaii_workspace();

        void init(struct ::wl_registry *registry, int id);
        void init(struct ::hawaii_workspace *object);

        struct ::hawaii_workspace *object() { return m_hawaii_workspace; }
        const struct ::hawaii_workspace *object() const { return m_hawaii_workspace; }

        bool isInitialized() const;

        void remove();

    protected:
        virtual void workspace_activated();
        virtual void workspace_deactivated();

    private:
        void init_listener();
        static const struct hawaii_workspace_listener m_hawaii_workspace_listener;
        static void handle_activated(
            void *data,
            struct ::hawaii_workspace *object);
        static void handle_deactivated(
            void *data,
            struct ::hawaii_workspace *object);
        struct ::hawaii_workspace *m_hawaii_workspace;
    };
}

QT_END_NAMESPACE

#endif
