/* 
 * Copyright (C) 2013-2014 Pier Luigi Fiorini
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 * 
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

extern const struct wl_interface hawaii_key_binding_interface;
extern const struct wl_interface hawaii_panel_interface;
extern const struct wl_interface hawaii_popup_surface_interface;
extern const struct wl_interface hawaii_window_interface;
extern const struct wl_interface hawaii_workspace_interface;
extern const struct wl_interface wl_output_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *types[] = {
	NULL,
	NULL,
	&hawaii_key_binding_interface,
	NULL,
	NULL,
	&wl_output_interface,
	&wl_surface_interface,
	&wl_output_interface,
	&wl_surface_interface,
	&wl_output_interface,
	&wl_surface_interface,
	&wl_surface_interface,
	&wl_surface_interface,
	&wl_surface_interface,
	&wl_surface_interface,
	NULL,
	NULL,
	&hawaii_workspace_interface,
	&wl_surface_interface,
	NULL,
	NULL,
	&hawaii_window_interface,
	NULL,
	NULL,
	NULL,
	&hawaii_window_interface,
	&hawaii_workspace_interface,
	NULL,
	&hawaii_panel_interface,
	&wl_surface_interface,
	NULL,
	&wl_output_interface,
	&hawaii_popup_surface_interface,
	&wl_surface_interface,
	&wl_surface_interface,
	NULL,
	NULL,
	&wl_output_interface,
	&wl_surface_interface,
};

static const struct wl_message hawaii_shell_requests[] = {
	{ "add_trusted_client", "hs", types + 0 },
	{ "add_key_binding", "nuu", types + 2 },
	{ "set_background", "oo", types + 5 },
	{ "set_desktop", "oo", types + 7 },
	{ "set_config_surface", "oo", types + 9 },
	{ "set_overlay", "o", types + 11 },
	{ "set_lock_surface", "o", types + 12 },
	{ "set_grab_surface", "o", types + 13 },
	{ "set_position", "oii", types + 14 },
	{ "quit", "", types + 0 },
	{ "lock", "", types + 0 },
	{ "unlock", "", types + 0 },
	{ "desktop_ready", "", types + 0 },
	{ "minimize_windows", "", types + 0 },
	{ "restore_windows", "", types + 0 },
	{ "add_workspace", "", types + 0 },
	{ "select_workspace", "o", types + 17 },
};

static const struct wl_message hawaii_shell_events[] = {
	{ "loaded", "", types + 0 },
	{ "configure", "oii", types + 18 },
	{ "prepare_lock_surface", "", types + 0 },
	{ "grab_cursor", "u", types + 0 },
	{ "window_mapped", "nssi", types + 21 },
	{ "window_switching_started", "", types + 0 },
	{ "window_switching_finished", "", types + 0 },
	{ "window_switched", "o", types + 25 },
	{ "workspace_added", "ni", types + 26 },
};

WL_EXPORT const struct wl_interface hawaii_shell_interface = {
	"hawaii_shell", 1,
	17, hawaii_shell_requests,
	9, hawaii_shell_events,
};

static const struct wl_message hawaii_panel_manager_requests[] = {
	{ "set_panel", "no", types + 28 },
};

WL_EXPORT const struct wl_interface hawaii_panel_manager_interface = {
	"hawaii_panel_manager", 1,
	1, hawaii_panel_manager_requests,
	0, NULL,
};

static const struct wl_message hawaii_panel_requests[] = {
	{ "set_alignment", "u", types + 0 },
	{ "set_offset", "u", types + 0 },
	{ "set_thickness", "u", types + 0 },
	{ "set_length", "u", types + 0 },
	{ "set_min_length", "u", types + 0 },
	{ "set_max_length", "u", types + 0 },
	{ "dock", "uo", types + 30 },
};

static const struct wl_message hawaii_panel_events[] = {
	{ "docked", "", types + 0 },
};

WL_EXPORT const struct wl_interface hawaii_panel_interface = {
	"hawaii_panel", 1,
	7, hawaii_panel_requests,
	1, hawaii_panel_events,
};

static const struct wl_message hawaii_shell_surface_requests[] = {
	{ "set_popup", "nooii", types + 32 },
	{ "set_dialog", "oo", types + 37 },
};

WL_EXPORT const struct wl_interface hawaii_shell_surface_interface = {
	"hawaii_shell_surface", 1,
	2, hawaii_shell_surface_requests,
	0, NULL,
};

static const struct wl_message hawaii_popup_surface_events[] = {
	{ "popup_done", "", types + 0 },
};

WL_EXPORT const struct wl_interface hawaii_popup_surface_interface = {
	"hawaii_popup_surface", 1,
	0, NULL,
	1, hawaii_popup_surface_events,
};

static const struct wl_message hawaii_key_binding_events[] = {
	{ "triggered", "", types + 0 },
};

WL_EXPORT const struct wl_interface hawaii_key_binding_interface = {
	"hawaii_key_binding", 1,
	0, NULL,
	1, hawaii_key_binding_events,
};

static const struct wl_message hawaii_window_requests[] = {
	{ "set_state", "i", types + 0 },
	{ "close", "", types + 0 },
};

static const struct wl_message hawaii_window_events[] = {
	{ "title_changed", "s", types + 0 },
	{ "identifier_changed", "s", types + 0 },
	{ "state_changed", "i", types + 0 },
	{ "unmapped", "", types + 0 },
};

WL_EXPORT const struct wl_interface hawaii_window_interface = {
	"hawaii_window", 1,
	2, hawaii_window_requests,
	4, hawaii_window_events,
};

static const struct wl_message hawaii_workspace_requests[] = {
	{ "remove", "", types + 0 },
};

static const struct wl_message hawaii_workspace_events[] = {
	{ "activated", "", types + 0 },
	{ "deactivated", "", types + 0 },
};

WL_EXPORT const struct wl_interface hawaii_workspace_interface = {
	"hawaii_workspace", 1,
	1, hawaii_workspace_requests,
	2, hawaii_workspace_events,
};

