#ifndef NOTIFICATION_SERVER_PROTOCOL_H
#define NOTIFICATION_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-util.h"

struct wl_client;
struct wl_resource;

struct wl_notification_daemon;

extern const struct wl_interface wl_notification_daemon_interface;

/**
 * wl_notification_daemon - interface for implementing notification
 *	daemons
 * @add_surface: add a notification surface
 *
 * Only one client can bind this interface at a time.
 *
 * Notification surfaces are special surfaces that the user should always
 * be able to see. The compositor is in charge of displaying them to be
 * visible without disturbing the user workflow.
 *
 * This interface must be bound by all notification daemons, ensuring that
 * only one can run at a time.
 */
struct wl_notification_daemon_interface {
	/**
	 * add_surface - add a notification surface
	 * @surface: (none)
	 *
	 * This request is for daemons which use a list of surfaces.
	 * Other mechanisms can be used and are not necessarily part of the
	 * wl_notification_daemon interface.
	 */
	void (*add_surface)(struct wl_client *client,
			    struct wl_resource *resource,
			    struct wl_resource *surface);
};


#ifdef  __cplusplus
}
#endif

#endif
