#ifndef NUCLEAR_SETTINGS_SERVER_PROTOCOL_H
#define NUCLEAR_SETTINGS_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-util.h"

struct wl_client;
struct wl_resource;

struct nuclear_settings;

extern const struct wl_interface nuclear_settings_interface;

#ifndef NUCLEAR_SETTINGS_MODIFIER_ENUM
#define NUCLEAR_SETTINGS_MODIFIER_ENUM
enum nuclear_settings_modifier {
	NUCLEAR_SETTINGS_MODIFIER_CTRL = 1,
	NUCLEAR_SETTINGS_MODIFIER_ALT = 2,
	NUCLEAR_SETTINGS_MODIFIER_SUPER = 4,
	NUCLEAR_SETTINGS_MODIFIER_SHIFT = 8,
};
#endif /* NUCLEAR_SETTINGS_MODIFIER_ENUM */

#ifndef NUCLEAR_SETTINGS_BINDING_TYPE_ENUM
#define NUCLEAR_SETTINGS_BINDING_TYPE_ENUM
enum nuclear_settings_binding_type {
	NUCLEAR_SETTINGS_BINDING_TYPE_KEY = 1,
	NUCLEAR_SETTINGS_BINDING_TYPE_BUTTON = 2,
	NUCLEAR_SETTINGS_BINDING_TYPE_AXIS = 4,
	NUCLEAR_SETTINGS_BINDING_TYPE_HOTSPOT = 8,
};
#endif /* NUCLEAR_SETTINGS_BINDING_TYPE_ENUM */

#ifndef NUCLEAR_SETTINGS_HOTSPOT_ENUM
#define NUCLEAR_SETTINGS_HOTSPOT_ENUM
enum nuclear_settings_hotspot {
	NUCLEAR_SETTINGS_HOTSPOT_TOP_LEFT_CORNER = 1,
	NUCLEAR_SETTINGS_HOTSPOT_TOP_RIGHT_CORNER = 2,
	NUCLEAR_SETTINGS_HOTSPOT_BOTTOM_LEFT_CORNER = 4,
	NUCLEAR_SETTINGS_HOTSPOT_BOTTOM_RIGHT_CORNER = 8,
};
#endif /* NUCLEAR_SETTINGS_HOTSPOT_ENUM */

struct nuclear_settings_interface {
	/**
	 * unset - (none)
	 * @path: (none)
	 * @option: (none)
	 */
	void (*unset)(struct wl_client *client,
		      struct wl_resource *resource,
		      const char *path,
		      const char *option);
	/**
	 * set_string - (none)
	 * @path: (none)
	 * @option: (none)
	 * @value: (none)
	 */
	void (*set_string)(struct wl_client *client,
			   struct wl_resource *resource,
			   const char *path,
			   const char *option,
			   const char *value);
	/**
	 * set_integer - (none)
	 * @path: (none)
	 * @option: (none)
	 * @value: (none)
	 */
	void (*set_integer)(struct wl_client *client,
			    struct wl_resource *resource,
			    const char *path,
			    const char *option,
			    int32_t value);
	/**
	 * set_key_binding - (none)
	 * @path: (none)
	 * @option: (none)
	 * @key: (none)
	 * @modifiers: (none)
	 */
	void (*set_key_binding)(struct wl_client *client,
				struct wl_resource *resource,
				const char *path,
				const char *option,
				uint32_t key,
				uint32_t modifiers);
	/**
	 * set_axis_binding - (none)
	 * @path: (none)
	 * @option: (none)
	 * @axis: (none)
	 * @modifiers: (none)
	 */
	void (*set_axis_binding)(struct wl_client *client,
				 struct wl_resource *resource,
				 const char *path,
				 const char *option,
				 uint32_t axis,
				 uint32_t modifiers);
	/**
	 * set_hotspot_binding - (none)
	 * @path: (none)
	 * @option: (none)
	 * @hotspot: (none)
	 */
	void (*set_hotspot_binding)(struct wl_client *client,
				    struct wl_resource *resource,
				    const char *path,
				    const char *option,
				    uint32_t hotspot);
	/**
	 * set_button_binding - (none)
	 * @path: (none)
	 * @option: (none)
	 * @button: (none)
	 * @modifiers: (none)
	 */
	void (*set_button_binding)(struct wl_client *client,
				   struct wl_resource *resource,
				   const char *path,
				   const char *option,
				   uint32_t button,
				   uint32_t modifiers);
};

#define NUCLEAR_SETTINGS_STRING_OPTION	0
#define NUCLEAR_SETTINGS_INTEGER_OPTION	1
#define NUCLEAR_SETTINGS_BINDING_OPTION	2

#define NUCLEAR_SETTINGS_STRING_OPTION_SINCE_VERSION	1
#define NUCLEAR_SETTINGS_INTEGER_OPTION_SINCE_VERSION	1
#define NUCLEAR_SETTINGS_BINDING_OPTION_SINCE_VERSION	1

static inline void
nuclear_settings_send_string_option(struct wl_resource *resource_, const char *path, const char *option)
{
	wl_resource_post_event(resource_, NUCLEAR_SETTINGS_STRING_OPTION, path, option);
}

static inline void
nuclear_settings_send_integer_option(struct wl_resource *resource_, const char *path, const char *option)
{
	wl_resource_post_event(resource_, NUCLEAR_SETTINGS_INTEGER_OPTION, path, option);
}

static inline void
nuclear_settings_send_binding_option(struct wl_resource *resource_, const char *path, const char *option, int32_t allowed_types)
{
	wl_resource_post_event(resource_, NUCLEAR_SETTINGS_BINDING_OPTION, path, option, allowed_types);
}

#ifdef  __cplusplus
}
#endif

#endif
